# -*- coding: utf-8 -*-
"""
Created on Sun Nov  6 13:16:22 2022

@author: richie bao

《环境景观——绿化种植设计》数据提取，写入SQLite数据库
"""
from database import df2SQLite, SQLite2df
import pandas as pd
import re

flatten_lst = lambda lst: (
    [m for n_lst in lst for m in flatten_lst(n_lst)]
    if type(lst) is list
    else [lst]
)


def gjbt599_text_extraction_A(txt_fn, s_mark, e_mark):
    """
    示例区段1，3文本文件格式信息提取方式，包括区段标识符和标题符，例如：
    ######1s
    $落叶灌木：
    木芙蓉(625)、木槿(626)、紫荆(402)
    $竹类：
    青皮竹(646)、慈竹(667)
    #######1e

    Parameters
    ----------
    txt_fn : string
        文本文档路径名.
    s_mark : string
        提取区段开始标识符，例如######1s，######3s等.
    e_mark : string
        提取区段结束标示符，例如#######1e，#######3e等.

    Returns
    -------
    veg_data_df : DataFrame
        返回提取的DataFrame格式数据，字段包含：index（为默认生成索引）、category（为标题转化）、name（为树种名提取）和idx（小括号内的树种索引）.

    """
    i = 0
    start = False
    key = ""
    veg_data = []
    mark = ""
    pattern_idx = re.compile("\((.+?)\)")
    pattern_name = re.compile("(.+?)\(")

    with open(txt_fn, "r", encoding="utf-8") as f:
        for line in f:
            line = line.strip()
            print(f"line------------------------------{i}", end="\r")
            if line == s_mark:
                start = True
                continue
            elif line == e_mark:
                break
            if start:
                # print(line)
                if line and line[0] == "$":
                    key = line[1:].split("：")[0]
                else:
                    line_split = line.split("、")
                    # print(line_split)
                try:
                    line_data = [
                        pattern_name.findall(i) + pattern_idx.findall(i)
                        for i in line_split
                    ]
                    if key and any(line_data):
                        line_data = [
                            [key, i[0].strip(), i[1].strip()] for i in line_data
                        ]
                        veg_data.extend(line_data)
                except:
                    pass
            i += 1
    veg_data_df = pd.DataFrame(veg_data, columns=["category", "name", "idx"])
    return veg_data_df


def gjbt599_text_extraction_B(txt_fn, s_mark, e_mark):
    """
    示例区段2文本文件格式信息提取方式，包括区段标识符，例如：
    ######2s
    1、小叶榕+榄树+朴树+假萍婆一散尾葵一春羽+艳山姜”的行数据
    12、火力楠+红花木莲一含笑+夜合+厚皮香一蚌花
    #######2e

    Parameters
    ----------
    txt_fn : string
        文本文档路径名.
    s_mark : string
        提取区段开始标识符，例如######2s，######5s等.
    e_mark : string
        提取区段结束标示符，例如#######2e，#######5e等.

    Returns
    -------
    veg_data_df : DataFrame
        返回提取的DataFrame格式数据，字段为数字索引，每一列为一组群落植被.

    """

    i = 0
    veg_data = []
    start = False
    with open(txt_fn, "r", encoding="utf-8") as f:
        for line in f:
            line = line.strip()
            print(f"line------------------------------{i}", end="\r")
            if line == s_mark:
                start = True
                continue
            elif line == e_mark:
                break
            if start:
                line_lst = re.split("一|—", line)
                line_lst = [i.split("、")[-1] for i in line_lst]
                line_lst = [i.split("+") for i in line_lst]
                line_lst = [
                    [f"{i}-" + j for j in line_lst[i]]
                    for i in range(len(line_lst))
                ]
                line_lst = flatten_lst(line_lst)
                veg_data.append(line_lst)
            i += 1
    veg_data_df = pd.DataFrame(veg_data).T
    return veg_data_df


if __name__ == "__main__":
    gjbt599_SQLite_fp = "../database/gjbt599_SQLite.sqlite"
    txt_fn = r"../data/vegetation_guangzhou.txt"

    A - Ⅶ区代表城市广州 - 常用园林植物及人工配置植物群落
    gjbt599_veg_text_zoning_representative = gjbt599_text_extraction_A(
        txt_fn, "######1s", "#######1e"
    )
    df2SQLite(
        gjbt599_SQLite_fp,
        gjbt599_veg_text_zoning_representative,
        "veg_zone_repr",
        method="replace",
    )

    B - 植物群里示例
    gjbt599_veg_biome = gjbt599_text_extraction_B(
        txt_fn, "######2s", "#######2e"
    )
    df2SQLite(
        gjbt599_SQLite_fp, gjbt599_veg_biome, "veg_biome", method="replace"
    )

    C - 生态性状
    gjbt599_veg_ecological_traits = gjbt599_text_extraction_A(
        txt_fn, "######3s", "#######3e"
    )
    df2SQLite(
        gjbt599_SQLite_fp,
        gjbt599_veg_ecological_traits,
        "veg_ecological_traits",
        method="replace",
    )

    # D-读取植物形状表
    veg_traits_table_fn = "../data/vegetation_traits_table.xlsx"
    veg_traits_table = pd.read_excel(veg_traits_table_fn, index_col=0)
    df2SQLite(
        gjbt599_SQLite_fp,
        veg_traits_table,
        "veg_traits_table",
        method="replace",
    )
